/*
 * ============================================================================
 * History
 * =======
 * 0.1 - date 2013.04.24 - Initial Version
 * 0.2 - date 2013.08.19 - few inclusions
 * 1.0 - date 2015.03.16 - Update with new APIs for FT81X
 *
 * (C) Copyright Bridgetek Pte Ltd
 * ============================================================================
 *
 * This source code ("the Software") is provided by Bridgetek Pte Ltd
 * ("Bridgetek") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of Bridgetek liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on Bridgetek's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, Bridgetek
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef FT_GPU_HAL_H
#define FT_GPU_HAL_H

typedef enum {
	FT_GPU_I2C_MODE = 0,
	FT_GPU_SPI_MODE,

	FT_GPU_MODE_COUNT,
	FT_GPU_MODE_UNKNOWN = FT_GPU_MODE_COUNT
}FT_GPU_HAL_MODE_E;

typedef enum {
	FT_GPU_HAL_OPENED,
	FT_GPU_HAL_READING,
	FT_GPU_HAL_WRITING,
	FT_GPU_HAL_CLOSED,

	FT_GPU_HAL_STATUS_COUNT,
	FT_GPU_HAL_STATUS_ERROR = FT_GPU_HAL_STATUS_COUNT
}FT_GPU_HAL_STATUS_E;

typedef struct {
	union {
		uint8_t spi_cs_pin_no;		//spi chip select number of ft8xx chip
		uint8_t i2c_addr;			//i2c address of ft8xx chip
	};
	union {
		uint16_t spi_clockrate_khz;  //In KHz
		uint16_t i2c_clockrate_khz;  //In KHz
	};
	uint8_t channel_no;				//mpsse channel number
	uint8_t pdn_pin_no;				//ft8xx power down pin number
}Ft_Gpu_Hal_Config_t;

typedef struct {
	uint8_t reserved;
}Ft_Gpu_App_Context_t;

typedef struct {
	/* Total number channels for libmpsse */
	uint32_t TotalChannelNum;
}Ft_Gpu_HalInit_t;

typedef enum {
	FT_GPU_READ = 0,
	FT_GPU_WRITE,
}FT_GPU_TRANSFERDIR_T;


typedef struct {	
	uint32_t length; //IN and OUT
	uint32_t address;
	uint8_t  *buffer;
}Ft_Gpu_App_Transfer_t;

typedef struct {
	Ft_Gpu_App_Context_t    app_header;
	Ft_Gpu_Hal_Config_t     hal_config;

    uint16_t 			ft_cmd_fifo_wp; //coprocessor fifo write pointer
    uint16_t 			ft_dl_buff_wp;  //display command memory write pointer

	FT_GPU_HAL_STATUS_E 	status;        //OUT
	void*          	hal_handle;        //IN/OUT
	/* Additions specific to ft81x */
	uint8_t				spichannel;			//variable to contain single/dual/quad channels
	uint8_t				spinumdummy;		//number of dummy bytes as 1 or 2 for spi read
}Ft_Gpu_Hal_Context_t;

/* FIFO buffer management */
typedef struct Ft_Fifo_t{
	uint32_t 		fifo_buff; 	//fifo buffer address
	int32_t 			fifo_len; 	//fifo length
	int32_t 			fifo_wp; 	//fifo write pointer - maintained by host
	int32_t 			fifo_rp;  	//fifo read point - maintained by device

	/* FT800 series specific registers */
	uint32_t 		HW_Read_Reg;	//hardware fifo read register
	uint32_t 		HW_Write_Reg;	//hardware fifo write register
}Ft_Fifo_t;

/*The basic APIs Level 1*/
int8_t              Ft_Gpu_Hal_Init(Ft_Gpu_HalInit_t *halinit);
int8_t              Ft_Gpu_Hal_Open(Ft_Gpu_Hal_Context_t *host);

/*The APIs for reading/writing transfer continuously only with small buffer system*/
void               Ft_Gpu_Hal_StartTransfer(Ft_Gpu_Hal_Context_t *host, FT_GPU_TRANSFERDIR_T rw, uint32_t addr);
uint8_t              Ft_Gpu_Hal_Transfer8(Ft_Gpu_Hal_Context_t *host, uint8_t value);
uint16_t             Ft_Gpu_Hal_Transfer16(Ft_Gpu_Hal_Context_t *host, uint16_t value);
uint32_t             Ft_Gpu_Hal_Transfer32(Ft_Gpu_Hal_Context_t *host, uint32_t value);
void               Ft_Gpu_Hal_EndTransfer(Ft_Gpu_Hal_Context_t *host);

/*Close and deinit apis*/
void              Ft_Gpu_Hal_Close(Ft_Gpu_Hal_Context_t *host);
void              Ft_Gpu_Hal_DeInit();

/*Helper function APIs Read*/
uint8_t  Ft_Gpu_Hal_Rd8(Ft_Gpu_Hal_Context_t *host, uint32_t addr);
uint16_t Ft_Gpu_Hal_Rd16(Ft_Gpu_Hal_Context_t *host, uint32_t addr);
uint32_t Ft_Gpu_Hal_Rd32(Ft_Gpu_Hal_Context_t *host, uint32_t addr);

/*Helper function APIs Write*/
void Ft_Gpu_Hal_Wr8(Ft_Gpu_Hal_Context_t *host, uint32_t addr, uint8_t v);
void Ft_Gpu_Hal_Wr16(Ft_Gpu_Hal_Context_t *host, uint32_t addr, uint16_t v);
void Ft_Gpu_Hal_Wr32(Ft_Gpu_Hal_Context_t *host, uint32_t addr, uint32_t v);

/*******************************************************************************/
/*******************************************************************************/
/*APIs for coprocessor Fifo read/write and space management*/
void Ft_Gpu_Hal_Updatecmdfifo(Ft_Gpu_Hal_Context_t *host, uint32_t count);
void Ft_Gpu_Hal_WrCmd32(Ft_Gpu_Hal_Context_t *host, uint32_t cmd);
void Ft_Gpu_Hal_WrCmdBuf(Ft_Gpu_Hal_Context_t *host, uint8_t *buffer, uint32_t count);
void Ft_Gpu_Hal_WaitCmdfifo_empty(Ft_Gpu_Hal_Context_t *host);
void Ft_Gpu_Hal_ResetCmdFifo(Ft_Gpu_Hal_Context_t *host);
void Ft_Gpu_Hal_CheckCmdBuffer(Ft_Gpu_Hal_Context_t *host, uint32_t count);

void Ft_Gpu_Hal_ResetDLBuffer(Ft_Gpu_Hal_Context_t *host);

void  Ft_Gpu_Hal_StartCmdTransfer(Ft_Gpu_Hal_Context_t *host, FT_GPU_TRANSFERDIR_T rw, uint16_t count);

void Ft_Gpu_Hal_Powercycle(Ft_Gpu_Hal_Context_t *host, int8_t up);


/*******************************************************************************/
/* APIs related to fifo buffer management */
/* API to update the read pointer into the structure */
void Ft_Fifo_Init(Ft_Fifo_t *pFifo, uint32_t StartAddress, uint32_t Length, uint32_t HWReadRegAddress, uint32_t HWWriteRegAddress);//Init all the parameters of fifo buffer
void Ft_Fifo_Update(Ft_Gpu_Hal_Context_t *host, Ft_Fifo_t *pFifo);//update both the read and write pointers
uint32_t Ft_Fifo_Write(Ft_Gpu_Hal_Context_t *host, Ft_Fifo_t *pFifo, uint8_t *buffer, uint32_t NumbytetoWrite);//just write and update the write register
void Ft_Fifo_Write32(Ft_Gpu_Hal_Context_t *host, Ft_Fifo_t *pFifo, uint32_t WriteWord);//just write one word and update the write register
void Ft_Fifo_WriteWait(Ft_Gpu_Hal_Context_t *host, Ft_Fifo_t *pFifo, uint8_t *buffer, uint32_t Numbyte);//write and wait for the fifo to be empty
uint32_t Ft_Fifo_GetFreeSpace(Ft_Gpu_Hal_Context_t *host, Ft_Fifo_t *pFifo);//get the free space in the fifo - make sure the return value is maximum of (LENGTH - 4)


/*******************************************************************************/
/*******************************************************************************/
/*APIs for Host Commands*/
typedef enum {
	FT_GPU_INTERNAL_OSC = 0x48, //default
	FT_GPU_EXTERNAL_OSC = 0x44,
}FT_GPU_PLL_SOURCE_T;


typedef enum {
	FT_GPU_PLL_48M = 0x62,  //default
	FT_GPU_PLL_36M = 0x61,
	FT_GPU_PLL_24M = 0x64,
}FT_GPU_PLL_FREQ_T;

typedef enum {
	FT_GPU_ACTIVE_M =  0x00,  
	FT_GPU_STANDBY_M = 0x41,//default
	FT_GPU_SLEEP_M =   0x42,
	FT_GPU_POWERDOWN_M = 0x50,
}FT_GPU_POWER_MODE_T;


#ifdef FT_81X_ENABLE
	typedef enum {
		FT_GPU_SYSCLK_DEFAULT = 0x00,  //default 60mhz
		FT_GPU_SYSCLK_72M = 0x06, 
		FT_GPU_SYSCLK_60M = 0x05,  
		FT_GPU_SYSCLK_48M = 0x04,  
		FT_GPU_SYSCLK_36M = 0x03,
		FT_GPU_SYSCLK_24M = 0x02,
	}FT_GPU_81X_PLL_FREQ_T;

	typedef enum{
		FT_GPU_MAIN_ROM = 0x80,  //main graphicas ROM used 
		FT_GPU_RCOSATAN_ROM = 0x40,  //line slope table used for 
		FT_GPU_SAMPLE_ROM = 0x20,  //JA samples
		FT_GPU_JABOOT_ROM = 0x10, //JA microcode
		FT_GPU_J1BOOT_ROM = 0x08, //J1 microcode
		FT_GPU_ADC = 0x01,  //
		FT_GPU_POWER_ON_ROM_AND_ADC = 0x00,  //specify this element to power on all ROMs and ADCs
	}FT_GPU_81X_ROM_AND_ADC_T;

	typedef enum{
		FT_GPU_5MA = 0x00,  //default current
		FT_GPU_10MA = 0x01,
		FT_GPU_15MA = 0x02,
		FT_GPU_20MA = 0x03,
	}FT_GPU_81X_GPIO_DRIVE_STRENGTH_T;

	typedef enum{
		FT_GPU_GPIO0 = 0x00,
		FT_GPU_GPIO1 = 0x04,
		FT_GPU_GPIO2 = 0x08,
		FT_GPU_GPIO3 = 0x0C,
		FT_GPU_GPIO4 = 0x10,
		FT_GPU_DISP = 0x20,
		FT_GPU_DE = 0x24,
		FT_GPU_VSYNC_HSYNC = 0x28,
		FT_GPU_PCLK = 0x2C,
		FT_GPU_BACKLIGHT = 0x30,
		FT_GPU_R_G_B = 0x34,
		FT_GPU_AUDIO_L = 0x38,
		FT_GPU_INT_N = 0x3C,
		FT_GPU_TOUCHWAKE = 0x40,
		FT_GPU_SCL = 0x44,
		FT_GPU_SDA = 0x48,
		FT_GPU_SPI_MISO_MOSI_IO2_IO3 = 0x4C,
	}FT_GPU_81X_GPIO_GROUP_T;

	#define FT_GPU_81X_RESET_ACTIVE 0x000268

	#define FT_GPU_81X_RESET_REMOVAL 0x002068
#endif

#define FT_GPU_CORE_RESET  (0x68)

/* Enums for number of SPI dummy bytes and number of channels */
typedef enum {
	FT_GPU_SPI_SINGLE_CHANNEL = 0,
	FT_GPU_SPI_DUAL_CHANNEL = 1,
	FT_GPU_SPI_QUAD_CHANNEL = 2,
}FT_GPU_SPI_NUMCHANNELS_T;
typedef enum {
	FT_GPU_SPI_ONEDUMMY = 1,
	FT_GPU_SPI_TWODUMMY = 2,
}FT_GPU_SPI_NUMDUMMYBYTES;

#define FT_SPI_ONE_DUMMY_BYTE	(0x00)
#define FT_SPI_TWO_DUMMY_BYTE	(0x04)
#define FT_SPI_SINGLE_CHANNEL	(0x00)
#define FT_SPI_DUAL_CHANNEL		(0x01)
#define FT_SPI_QUAD_CHANNEL		(0x02)

int32_t hal_strlen(const int8_t *s);
void Ft_Gpu_Hal_Sleep(uint32_t ms);
void Ft_Gpu_ClockSelect(Ft_Gpu_Hal_Context_t *host, FT_GPU_PLL_SOURCE_T pllsource);
void Ft_Gpu_PLL_FreqSelect(Ft_Gpu_Hal_Context_t *host, FT_GPU_PLL_FREQ_T freq);
void Ft_Gpu_PowerModeSwitch(Ft_Gpu_Hal_Context_t *host, FT_GPU_POWER_MODE_T pwrmode);
void Ft_Gpu_CoreReset(Ft_Gpu_Hal_Context_t *host);
void Ft_Gpu_Hal_StartTransfer(Ft_Gpu_Hal_Context_t *host, FT_GPU_TRANSFERDIR_T rw, uint32_t addr);
void Ft_Gpu_Hal_WrMem(Ft_Gpu_Hal_Context_t *host, uint32_t addr, const uint8_t *buffer, uint32_t length);
void Ft_Gpu_Hal_WrMemFromFlash(Ft_Gpu_Hal_Context_t *host, uint32_t addr, const ft_prog_uint8_t *buffer, uint32_t length);
void Ft_Gpu_Hal_WrCmdBufFromFlash(Ft_Gpu_Hal_Context_t *host, FT_PROGMEM ft_prog_uint8_t *buffer, uint32_t count);
void Ft_Gpu_Hal_RdMem(Ft_Gpu_Hal_Context_t *host, uint32_t addr, uint8_t *buffer, uint32_t length);
void Ft_Gpu_Hal_WaitLogo_Finish(Ft_Gpu_Hal_Context_t *host);
uint8_t Ft_Gpu_Hal_TransferString(Ft_Gpu_Hal_Context_t *host, const char *string);
void Ft_Gpu_HostCommand(Ft_Gpu_Hal_Context_t *host, uint8_t cmd);
void Ft_Gpu_HostCommand_Ext3(Ft_Gpu_Hal_Context_t *host, uint32_t cmd);
int32_t Ft_Gpu_Hal_Dec2Ascii(char *pSrc, int32_t value);
uint8_t Ft_Gpu_Hal_WaitCmdfifo_empty_status(Ft_Gpu_Hal_Context_t *host);
void Ft_Gpu_Hal_WrCmdBuf_nowait(Ft_Gpu_Hal_Context_t *host, uint8_t *buffer, uint32_t count);
uint16_t Ft_Gpu_Cmdfifo_Freespace(Ft_Gpu_Hal_Context_t *host);
int16_t Ft_Gpu_Hal_SetSPI(Ft_Gpu_Hal_Context_t *host, FT_GPU_SPI_NUMCHANNELS_T numchnls, FT_GPU_SPI_NUMDUMMYBYTES numdummy);

#ifdef FT_81X_ENABLE
void Ft_Gpu_81X_SelectSysCLK(Ft_Gpu_Hal_Context_t *host, FT_GPU_81X_PLL_FREQ_T freq);
void Ft_GPU_81X_PowerOffComponents(Ft_Gpu_Hal_Context_t *host, uint8_t val);
void Ft_GPU_81X_PadDriveStrength(Ft_Gpu_Hal_Context_t *host, FT_GPU_81X_GPIO_DRIVE_STRENGTH_T strength, FT_GPU_81X_GPIO_GROUP_T group);
void Ft_Gpu_81X_ResetActive(Ft_Gpu_Hal_Context_t *host);
void Ft_Gpu_81X_ResetRemoval(Ft_Gpu_Hal_Context_t *host);
#endif

uint32_t Ft_Gpu_CurrentFrequency(Ft_Gpu_Hal_Context_t *host);
int32_t Ft_Gpu_ClockTrimming(Ft_Gpu_Hal_Context_t *host, int32_t LowFreq);

#endif  /* FT_GPU_HAL_H */
