/**
 @file FT_Platform.h
 */
/*
 * ============================================================================
 * History
 * =======
 * 2017-03-15 : Created
 *
 * (C) Copyright Bridgetek Pte Ltd
 * ============================================================================
 *
 * This source code ("the Software") is provided by Bridgetek Pte Ltd
 * ("Bridgetek") subject to the licence terms set out
 * http://www.ftdichip.com/FTSourceCodeLicenceTerms.htm ("the Licence Terms").
 * You must read the Licence Terms before downloading or using the Software.
 * By installing or using the Software you agree to the Licence Terms. If you
 * do not agree to the Licence Terms then do not download or use the Software.
 *
 * Without prejudice to the Licence Terms, here is a summary of some of the key
 * terms of the Licence Terms (and in the event of any conflict between this
 * summary and the Licence Terms then the text of the Licence Terms will
 * prevail).
 *
 * The Software is provided "as is".
 * There are no warranties (or similar) in relation to the quality of the
 * Software. You use it at your own risk.
 * The Software should not be used in, or for, any medical device, system or
 * appliance. There are exclusions of Bridgetek liability for certain types of loss
 * such as: special loss or damage; incidental loss or damage; indirect or
 * consequential loss or damage; loss of income; loss of business; loss of
 * profits; loss of revenue; loss of contracts; business interruption; loss of
 * the use of money or anticipated savings; loss of information; loss of
 * opportunity; loss of goodwill or reputation; and/or loss of, damage to or
 * corruption of data.
 * There is a monetary cap on Bridgetek's liability.
 * The Software may have subsequently been amended by another user and then
 * distributed by that other user ("Adapted Software").  If so that user may
 * have additional licence terms that apply to those amendments. However, Bridgetek
 * has no liability in relation to those amendments.
 * ============================================================================
 */

#ifndef _FT_PLATFORM_H_
#define _FT_PLATFORM_H_

/** @brief Target FT9xx Platform
 */
//@{
#define MM900EV1A		1
#define MM900EV2A		2
#define MM900EV3A		3
#define MM900EV_LITE	4

#ifndef FT900_PLATFORM
#define FT900_PLATFORM	MM900EV2A
#endif // FT900_PLATFORM
//@}

/** @brief Target FT8xx Platform
 */
//@{
#define ME800A_HV35R	1
#define ME810A_HV35R	2
#define ME812A_WH50R	3
#define ME813A_WH50C	4

#ifndef FT800_PLATFORM
#define FT800_PLATFORM 	ME812A_WH50R
#endif
//@}

/** @brief Module specific device configurations
 */
//@{
#define FT_800_ENABLE	1
#define FT_801_ENABLE	2
#define FT_810_ENABLE	3
#define FT_811_ENABLE	4
#define FT_812_ENABLE	5
#define FT_813_ENABLE	6
//@}

/** @brief Module specific display configurations
 */
//@{
#define DISPLAY_RESOLUTION_HVGA_PORTRAIT 	1
#define DISPLAY_RESOLUTION_WVGA				2
#define DISPLAY_RESOLUTION_QVGA				3
#define DISPLAY_RESOLUTION_WQVGA			4
//@}

/** @brief Module specific SPI configurations
 */
//@{
#define ENABLE_SPI_SINGLE	1
#define ENABLE_SPI_DUAL		2
#define ENABLE_SPI_QUAD		4
//@}

/** @brief Module specific configurations
 */
//@{
#if FT800_PLATFORM == ME800A_HV35R

#define SPI_ENABLE						ENABLE_SPI_SINGLE
#define FT800_DEVICE					FT_800_ENABLE
#define DISPLAY_RESOLUTION				DISPLAY_RESOLUTION_HVGA_PORTRAIT
#define ENABLE_ILI9488_HVGA_PORTRAIT	1
#define RESISTANCE_THRESHOLD			1800

#elif FT800_PLATFORM ==  ME810A_HV35R

#define SPI_ENABLE						ENABLE_SPI_QUAD
#define FT800_DEVICE					FT_810_ENABLE
#define DISPLAY_RESOLUTION				DISPLAY_RESOLUTION_HVGA_PORTRAIT
#define ENABLE_ILI9488_HVGA_PORTRAIT	1
#define RESISTANCE_THRESHOLD			1800

#elif FT800_PLATFORM == ME812A_WH50R

#define SPI_ENABLE						ENABLE_SPI_QUAD
#define FT800_DEVICE					FT_812_ENABLE
#define DISPLAY_RESOLUTION				DISPLAY_RESOLUTION_WVGA
#define RESISTANCE_THRESHOLD			1800

#elif FT800_PLATFORM == ME813A_WH50C

#define SPI_ENABLE						ENABLE_SPI_QUAD
#define FT800_DEVICE					FT_813_ENABLE
#define DISPLAY_RESOLUTION				DISPLAY_RESOLUTION_WVGA

#else /* FT800_PLATFORM */

/* Individual configurations if module specific configurations are not defined */
/* Enable the respective macros based on the custom platform */

/* Display configuration specific macros */
#define DISPLAY_RESOLUTION				DISPLAY_RESOLUTION_HVGA_PORTRAIT

/* Chip configuration specific macros */
#define FT800_DEVICE					FT_810_ENABLE

/* SPI specific macros - compile time switches for SPI single, dial and quad use cases */
#define SPI_ENABLE						ENABLE_SPI_SINGLE

/* Display driver configurations - mainly for ME900EV1 modules */
#define ENABLE_ILI9488_HVGA_PORTRAIT	1

#endif /* FT800_PLATFORM */
//@}

#if (defined(FT_800_ENABLE) || defined(FT_801_ENABLE))
#define FT_80X_ENABLE					1
#endif

#if (defined(FT_810_ENABLE) || defined(FT_811_ENABLE) \
		|| defined(FT_812_ENABLE) || defined(FT_813_ENABLE))
#define FT_81X_ENABLE					1
#endif



/* Standard C libraries */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>

/* FT900 libraries */
#include <ft900.h>

/* HAL inclusions */
#include "FT_DataTypes.h"
#include "FT_Gpu_Hal.h"
#include "FT_Gpu.h"
#include "FT_CoPro_Cmds.h"
#include "FT_Hal_Utils.h"
#include "FT_ILI9488.h"

/* Hardware or Module specific macros for gpio line numbers  */
#if (FT900_PLATFORM == MM900EV1A) || (FT900_PLATFORM == MM900EV2A) \
	|| (FT900_PLATFORM == MM900EV3A) || (FT900_PLATFORM == MM900EV_LITE)
#define FT800_SEL_PIN   0
#define FT800_PD_N      43

/* Timer 1 is been utilized in case of FT900 platform */
#define	FT900_FT_MILLIS_TIMER					(timer_select_b)
#define FT900_TIMER_MAX_VALUE 					(65536L)
#define FT900_TIMER_PRESCALE_VALUE 				(100)
#define FT900_TIMER_OVERFLOW_VALUE 				(1000)

#endif // FT900_PLATFORM

void ft_millis_init();
void millis_ticker();

#endif /*_FT_PLATFORM_H_*/
/* Nothing beyond this*/

