"use strict";
var sys = require("system");
var fs = require("fs");
//var process = require("child_process");
var page = require("webpage").create();
//var childExec = require("child_process").childExec;
var spawn = require("child_process").spawn;
//var execFile = require("child_process").execFile

// Parameter Manipulation
var arg_count = sys.args.length;
var arg_next = 1;

// Parameter Default Values
var url = "";
var fileType = "";
var sizeX = 800, sizeY = 480;
var d2xx = "";
var d2xxall = 0;
var	useStdout = 1;
var prevFileName = "";
var continuousPoll = 1;
var progressPoll = 1;
var logOutput = 0;
var logResources = 0;
var overwriteFiles = 1;
var uniqueFileName = 0;
var prefixFile = "active";
var pathToTty = "ftttyusb.exe";

// Global Variables
var currentFileName;
var resourceWait  = 300;
var periodicWait  = 250;
var overallWait = 10000;
var	fileStdout = "";
var count = 0;
var overallTimeout;
var resourceTimeout;
var periodicTimeout;
var child;
var sys_eol;

// Argument setup
if (arg_count < 2) 
{
	arg_error(1);
}

while (arg_next < arg_count)
{
	arg_parse();
	arg_next++;
	if (arg_next > arg_count) arg_error(1);
}

// Argument checks and defaults.
if (url == "")
{
	console.log("No URL specified");
	phantom.exit(1);
}

if (fileType == "")
{
	if (logOutput > 0)
	{
		console.log("Default file type JPEG");
	}
	fileType = "JPEG";
}

if (!useStdout)
{
	currentFileName = prefixFile + "." + fileType;
	fs.touch(currentFileName);
}

if (d2xx == "")
{
	console.log("No D2XX device specified");
	phantom.exit(1);
}
else
{
	if (logOutput > 0)
	{
		console.log("Opening " + d2xx);
	}
	child = spawn(pathToTty, ["-p", d2xx, "-m", "-f", "active.JPEG", "-o", "-"]);
	
	child.stdout.on("data", function (data) {
		var stdoutStr = data;
		var coordLines = stdoutStr.split(sys_eol);
		for (var i = 0, len = coordLines.length; i < len; i++) {
			var coordPts = coordLines[i].split(' ');
			if (logOutput > 0)
			{
				console.log("X: " + coordPts[0]);
				console.log("Y: " + coordPts[1]);
			}
			page.sendEvent('mousedown', coordPts[0], coordPts[1], button='left');
			page.sendEvent('mouseup', coordPts[0], coordPts[1], button='left');
		}
	})

	child.stderr.on("data", function (data) {
	  console.log("spawnSTDERR:", JSON.stringify(data))
	})

	child.on("exit", function (code) {
	  console.log("spawnEXIT:", code)
	  phantom.exit(0);
	})
}

// Platform setup
if (sys.os.name == 'windows')
{
	if (logOutput > 0)
	{
		console.log("Windows");
	}
	fileStdout = "\\\\.\\CON";
	sys_eol = "\r\n";
}
else
{
	if (logOutput > 0)
	{
		console.log("Linux/OSX");
	}
	fileStdout = "/dev/stdout";
	sys_eol = "\n";
}

// Page setup
page.viewportSize = { width: sizeX, height: sizeY };
page.clipRect = { top: 0, left: 0, width: sizeX,  height: sizeY };

if (logOutput > 0)
{
	console.log("Opening: "+url);
}

// Page open	
page.open(url, function (status) 
{
	if (logOutput > 0)
	{
		console.log("Loaded page: " + url);
	}
	
	if (status !== "success") 
	{
		console.log("Cannot load page: " + url);
		phantom.exit(2);
	} 
	else 
	{
		if (progressPoll)
		{
			startPeriodicPoll();
		}
		        
		if (!continuousPoll)
		{
	        overallTimeout = setTimeout(function () 
	        {
	            doRender();
				if (logOutput > 0)
				{
		            console.log("Page timeout");
				}
				phantom.exit(1);
				            
			}, overallWait);
		}
	}
});

// Argument error - show help information
function arg_error(ret) {
	console.log("Usage: " + sys.args[0] + " [OPTION] URL");
	console.log("       -p d2xx   D2XX device name");
	console.log("       -a        List available devices");
	console.log("       -u exe    Location of \"ftttyusb\" program");
	console.log("       -t ms     Timer (in ms) for writing output during load");
	console.log("                 Default is " + periodicWait + " ms.");
	console.log("       -q        Quieten output");
	console.log("       -l        Print filename when writing output files (default)");
	console.log("       -la       Print when output file is not overridden");
	console.log("       -r        Report resources loaded");
	console.log("       -x width  Output width (default is "+sizeX+")");
	console.log("       -y height Output height (default is "+sizeY+")");
	phantom.exit(ret);
}

// Parse arguments
function arg_parse() {

	if (sys.args[arg_next] == "--help")
	{
			arg_error(0);
	}
	else if (sys.args[arg_next] == "-p")
	{
		arg_next++;
		if (arg_next > arg_count)
		{
			arg_error(1);
			console.log("-p requires a parameter");
		}
		d2xx = sys.args[arg_next];
		useStdout = 0;
	}
	else if (sys.args[arg_next] == "-a")
	{
		d2xxall = 1;
	}
	else if (sys.args[arg_next] == "-u")
	{
		arg_next++;
		if (arg_next > arg_count)
		{
			arg_error(1);
			console.log("-u requires a parameter");
		}
		pathToTty = sys.args[arg_next];
	}
	else if (sys.args[arg_next] == "-q")
	{
		logOutput = 0;
	}
	else if (sys.args[arg_next] == "-l")
	{
		logOutput = 1;
	}
	else if (sys.args[arg_next] == "-la")
	{
		logOutput = 2;
	}
	else if (sys.args[arg_next] == "-r")
	{
		logResources = 1;
	}
	else if (sys.args[arg_next] == "-t")
	{
		arg_next++;
		if (arg_next > arg_count)
		{
			arg_error(1);
			console.log("-t requires a parameter");
		}
		periodicWait = sys.args[arg_next];
	}
	else if (sys.args[arg_next] == "-x")
	{
		arg_next++;
		if (arg_next > arg_count) arg_error(1);
		sizeX = sys.args[arg_next];
	}
	else if (sys.args[arg_next] == "-y")
	{
		arg_next++;
		if (arg_next > arg_count) arg_error(1);
		sizeY = sys.args[arg_next];
	}
	else
	{
		if (url == "")
		{
			url = sys.args[arg_next];
		}
		else if (fileType == "")
		{
			fileType = sys.args[arg_next];
		}
		else
		{
			arg_error(1);
		}
	}
}

function startPeriodicPoll() 
{
	periodicTimeout = setTimeout(function () {
	    doRender();
	    startPeriodicPoll();
	}, periodicWait);
}

function doRender() 
{
	currentFileName = prefixFile;

	if (useStdout)
	{
		if (logOutput > 0)
		{
			console.log("stdout (" + fileStdout + ") with type " + fileType);
	    }
	    page.render(fileStdout, { format: fileType});
	}
	else
	{	
		if (uniqueFileName)
		{
			if (prefixFile != "")
			{
				currentFileName += "_";
			}
			currentFileName += Date.now().toString() + "." + fileType;
		}
		else if (prevFileName == "")
		{
			currentFileName = prefixFile + "." + fileType;
		}
		else
		{
			currentFileName = prevFileName;
		}
		
		if (logOutput > 0)
		{
			console.log("Rendering: " + currentFileName);
		}
		
	    if ((!fs.exists(prevFileName)) || (overwriteFiles))
	    {
		    page.render(currentFileName, fileType);
			if (logOutput > 0)
			{
				console.log("Rendered to file: " + currentFileName);
		    }
		    prevFileName = currentFileName;
	    }
	    else
	    {
			if (logOutput > 1)
			{
				console.log("Did not write: " + currentFileName);
		    }
		}
	}
}

page.onResourceRequested = function (req) 
{
	count += 1;
	if (logResources)
	{
	    console.log("> " + req.id + " - " + req.url);
	}
	clearTimeout(resourceTimeout);
};

page.onResourceReceived = function (res) 
{
	if (!res.stage || res.stage === "end") {
		count -= 1;
		if (logResources)
		{
			console.log(res.id + " " + res.status + " - " + res.url);
		}
		if (count === 0) {
			resourceTimeout = setTimeout(doRender, resourceWait);
		}
	}
};

page.onLoadFinished = function() {
	doRender();
	if (logOutput > 0)
	{
		console.log("Page Loaded");
	}
	if (continuousPoll == 0)
	{
		phantom.exit(0);
	}
};
